@define-color button_color1 #444c5b;
@define-color button_color2 darker(@button_color1);
@define-color keyboard_color1 #000e29;
@define-color keyboard_color2 darker(#000e29);
@define-color key_text_color #ffffff;
@define-color insensitive_color #888888;
@define-color key_scan_color1 #dd0000;
@define-color key_scan_color2 darker(@key_scan_color1);
@define-color group_scan_color1 #00aa00;
@define-color group_scan_color2 darker(@group_scan_color1);

.antler-keyboard-button {
  border-width: 0px;
  border-radius: 2px;
  border-image: url("dark-key-border.svg") 2 2 2 2 repeat stretch;
  font: 14px Sans;
  background-image: -gtk-gradient (linear,
				     left top,
				     left bottom,
				     from (@button_color1),
				     to (@button_color2));
  color: @key_text_color;
}

.antler-keyboard-button:hover {
  background-image: -gtk-gradient (linear,
				     left top,
				     left bottom,
				     from (shade(@button_color1, 1.1)),
				     to (shade(@button_color2, 1.1)));
}

.antler-keyboard-button:active, .antler-keyboard-button:hover:active {
  background-image: -gtk-gradient (linear,
				     left top,
				     left bottom,
				     from (shade(@button_color2, 1.4)),
				     to (shade(@button_color1, 1.4)));
}

.antler-keyboard-button:disabled {
  background-image: -gtk-gradient (linear,
				     left top,
				     left bottom,
				     from (shade(@button_color1, 0.5)),
				     to (shade(@button_color2, 0.5)));
  color: #aaaaaa;
}

.antler-keyboard-button:selected,
.antler-keyboard-button:selected:indeterminate { /* scanning */
  background-image: -gtk-gradient (linear,
                                     left top,
                                     left bottom,
                                     from (@key_scan_color1),
                                     to (@key_scan_color2));
}

.antler-keyboard-button:selected:active {
  background-image: -gtk-gradient (linear,
				     left top,
				     left bottom,
				     from (shade(@key_scan_color2, 1.4)),
				     to (shade(@key_scan_color1, 1.4)));
}

.antler-keyboard-button:indeterminate { /* scanning */
  background-image: -gtk-gradient (linear,
                                     left top,
                                     left bottom,
                                     from (@group_scan_color1),
                                     to (@group_scan_color2));
}

.antler-keyboard-window {
background-image: -gtk-gradient (linear,
  left top,
  left bottom,
  from (@keyboard_color1),
  to (@keyboard_color2));
}

